﻿/*
Copyright (c) 2009 Hideo KINAMI

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

// IntentActivity.java
package kr.hews.intent;

import android.app.Activity;
import android.content.ContentUris;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.provider.Contacts;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;

public class IntentActivity extends Activity implements OnClickListener {
	
    static final int[] BUTTONS = {
        R.id.action_dial,
        R.id.action_edit_contacts,
        R.id.action_view_geo,
        R.id.action_view_http,
        R.id.start_activity,
        R.id.start_activity_for_result
    };
	
    // 개발용 Log 태그
    private static final String TAG = "IntentActivity";
	
	
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        // 버튼 등록
        for(int buttonId: BUTTONS) {
            Button button = (Button)findViewById(buttonId);
            button.setOnClickListener(this);
        }
    }
    
    // 눌러진 버튼에 해당하는 메소드를 호출
    public void onClick(View v) {
        int id = v.getId();
        if (id == R.id.action_dial) {
            sendDialIntent();
        } else if (id == R.id.action_edit_contacts) {
            sendEditContacts();
        } else if (id == R.id.action_view_geo) {
            sendViewGeo();
        } else if (id == R.id.action_view_http) {
            sendViewHttp();
        } else if (id == R.id.start_activity) {
            sendStartActivity();
        } else if (id == R.id.start_activity_for_result) {
            sendActivityForResult();
        } else {
        	Log.e(TAG, "Unkowon Button:"+v.toString());
        }
    }

    // 전화 화면 작업을 시작
    private void sendDialIntent() {
    	Intent i = new Intent(Intent.ACTION_DIAL,
                              Uri.parse("tel:1234567890"));
    	startActivity(i);	
    }

    // 연락처 목록 id 가장 편집 작업을 시작
    private void sendEditContacts() {
        // 일반적으로 주소록 검색에서 얻은 Uri 사용
        Uri uri = ContentUris.withAppendedId(
            Contacts.People.CONTENT_URI, 1);
    	Intent i = new Intent(Intent.ACTION_EDIT,uri);
    	startActivity(i);
    }

    // 지도 보기 활동을 시작
    private void sendViewGeo() {
        Intent i = new Intent(Intent.ACTION_VIEW,
                              Uri.parse("geo:0,0"));
    	startActivity(i);	
    }

    // 브라우저를 표시하는 작업을 시작
    private void sendViewHttp() {
        Intent i = new Intent(Intent.ACTION_VIEW,
                              Uri.parse("http://www.google.com/"));
    	startActivity(i);	
    }

    // 동일한 응용 프로그램에서 활동을 시작
    private void sendStartActivity() {
        Intent intent = new Intent(getApplicationContext(),
                                   LaunchedActivity.class);
        startActivity(intent);	
    }

 // 활동 시작 요청 코드
    private static final int LAUNCHED_ACTIVITY = 1;

    // 활동을 시작
    private void sendActivityForResult() {
        Intent intent = new Intent(getApplicationContext(),
                                   LaunchedActivity.class);
        // 두번째 인자로 시작 요청 코드를 지정
        startActivityForResult(intent, LAUNCHED_ACTIVITY);
    }

    @Override
    protected void onActivityResult(int requestCode,
                                    int resultCode,
                                    Intent data) {
        // 돌아온 시작 요청 코드에서 어떤 작업이 종료되었는지 확인
        if (requestCode == LAUNCHED_ACTIVITY) {
            TextView textView = (TextView)findViewById(R.id.result);
            if (resultCode != RESULT_OK) {
                // 결과가 OK 아닌 경우 오류를 표시
                textView.setText("Error.");
            } else {
                // 설정된 텍스트를 검색
                String result = data.getExtras().getString(
                    LaunchedActivity.TEXT_RESULT);
                // 텍스트를 텍스트 뷰에서보기
                textView.setText(result);
            }
        } else {
            super.onActivityResult(requestCode, resultCode, data);
        }
    }
}

