﻿/*
   Copyright 2009 Hideo KINAMI

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

// LaunchedActivity.java
package kr.hews.intent;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;

// startActivityForResult 메소드에 의해 호출되는 Activity
public class LaunchedActivity extends Activity {

    // 실행 시에 인텐트에 전달된 데이터 키
    public static final String TEXT_INPUT = "TextInput";
    // 종료 시에 인텐트에 전달된 데이터 키
    public static final String TEXT_RESULT = "TextResult";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.launched_view);
		
        // 인텐트로부터 Parameter를 읽어 드림
        String input = getIntent().getStringExtra(TEXT_INPUT);
        if (input != null) {
            EditText edit = (EditText)findViewById(R.id.result_text);
            edit.setText(input);
        }

        Button finish = (Button)findViewById(R.id.finish_button);
        finish.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                // 버튼을 누르면 종료 
                callFinish();
            }});
    }

    // 텍스트 박스의 내용을 인텐트로 설정하고 Activity를 종료
    private void callFinish() {
        EditText edit = (EditText)findViewById(R.id.result_text);
        String result = edit.getText().toString();
        if (result.length() != 0) {
            // 텍스트 박스에 문자가 입력되어 있으면 인텐트로 설정
            Intent i = new Intent();
            i.putExtra(TEXT_RESULT, result);
            // 결과 코드(RESULT_OK)와 함께 인텐트를 실행 이전으로 되돌림
            setResult(RESULT_OK, i);
        } else {
            // 텍스트박스에 문자가 입력되어 있지 않으면 결과는 취소(cancel)
            setResult(RESULT_CANCELED);
        }
        // Activity를 종료
        finish();
    }
}
