﻿/*
Copyright (c) 2009 Hideo KINAMI

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

/*
  그림 맞추기 퍼즐
*/
package kr.hews.picpuzzle;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.SystemClock; 
import android.view.View;
import android.widget.Button;
import android.widget.Chronometer;
import android.widget.ImageButton;

// 패널 게임 화면 제어 클래스
public class PicturePuzzle extends Activity {

    // （1）패널 구성 버튼
    static final int mImageButtons[] = {
        R.id.image_button1, R.id.image_button2, R.id.image_button3,
        R.id.image_button4, R.id.image_button5, R.id.image_button6,
        R.id.image_button7, R.id.image_button8, R.id.image_button9
    };

    // （2）패널에 표시되는 이미지의 초기 상태
    static final int mInitialImages[] = {
        R.drawable.image1, R.drawable.image2, R.drawable.image3,
        R.drawable.image4, R.drawable.image5, R.drawable.image6,
        R.drawable.image7, R.drawable.image8, R.drawable.blank_image
    };

    /* （3）교체 가능한 (인접한) 패널의 인덱스
        예를 들어 0번(왼쪽 상단)의 패널은 1번(중앙 상단)이 패널과
        3번(좌측 중앙)의 패널과 인접하고 있다는 것을 (1,3)로 표시
    */
    static final int mNextPanelIndex[][] = {
        {1,3},   {0,4,2},   {1,5},
        {0,4,6}, {1,3,5,7}, {2,4,8},
        {3,7},   {4,6,8},   {7,5}
    };
        
    // （4）버튼을 제어하는 컨트롤러의 배열
    PanelController mPanels[] = 
        new PanelController[mImageButtons.length];

    // 버튼을 제어하는 컨트롤러
    class PanelController implements View.OnClickListener {
        ImageButton mButton;    // 제어 버튼
        int mCurrentImage = 0;  // 표시된 이미지 리소스 ID
        final int mNext[];      // 움직일 수있는 패널

        // 제어 버튼, 이미지, 교환할 수있는 패널을 지정
        public PanelController(ImageButton button,
                               int resid,
                               final int next[]) {
            mButton = button;
            mNext = next;
            setImageResource(resid);
            mButton.setOnClickListener(this);
        }
        
        // （5）새 이미지 리소스 ID를 설정
        // 현재 설정된 이미지 리소스 ID를 반환
        public int setImageResource(int resid) {
            int old = mCurrentImage;
            mCurrentImage = resid;
            mButton.setImageResource(resid);
            return old;
        }
        
        // 현재 설정된 이미지 리소스 ID를 반환
        public int getImageResource() {
            return mCurrentImage;
        }
        
        // 보고있는 것이 빈 화면 경우 true를 반환
        public boolean isBlank() {
            return mCurrentImage == R.drawable.blank_image;
        }
        
        // （6）다른 패널과 사진 교환
        public void swapImage(PanelController other) {
            int previous = other.setImageResource(mCurrentImage);
            setImageResource(previous);
        }

        // （7）버튼 탭 동작
        public void onClick(View v) {
            for(int i=0; i<mNext.length; ++i) {
        // 인접한 패널이 빈 화면 경우 이미지를 교체
                if (mPanels[mNext[i]].isBlank()) {
                    swapImage(mPanels[mNext[i]]);
                    break;
                } 
            }
            if (isCompleted())
                complete();
        }
    }

    // （8）프로그램 시작시 호출되는 메서드
    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        setContentView(R.layout.main);
        createPanelController();
        setShuffleButtonListener();
    }
    
    // （9）각 버튼에 PanelController 등록
    private void createPanelController() {
        for(int i=0; i<mImageButtons.length; ++i) {
            ImageButton b = 
                (ImageButton)findViewById(mImageButtons[i]);
            mPanels[i] = new PanelController(b,
                                             mInitialImages[i],
                                             mNextPanelIndex[i]);
        }
    }
    
    // （10）패널의 이미지가 처음 상태와 일치하면 완료로 확인
    private boolean isCompleted() {
        for(int i=0; i<mInitialImages.length; ++i) {
            if (mInitialImages[i] != mPanels[i].getImageResource())
                return false;
        }
        return true;
    }

    // （11）패널을 섞음
    private void shuffle() {
        int size = mInitialImages.length;
        for(int i=0; i<size-1; ++i) {
            int swap = (int)(Math.random()*(size-i));
            mPanels[i].swapImage(mPanels[i+swap]);
        }
    }

    // （12）현재 시간을 설정하고 타이머를 시작
    private void startChronometer() {
        Chronometer c =
            (Chronometer)findViewById(R.id.chronometer);
        c.setBase(SystemClock.elapsedRealtime());
        c.start();
    }
    
    // （13）타이머를 중지하고 경과 시간을 반환
    private long stopChronometer() {
        Chronometer c =
            (Chronometer)findViewById(R.id.chronometer);
        c.stop();
        return SystemClock.elapsedRealtime() - c.getBase();
    }
    
    // （14）Shuffle Button을 눌렀을 때의 동작
    private void setShuffleButtonListener() {
        // Shuffle Button을 불러낸다
        Button b = (Button)findViewById(R.id.shuffle_button);
        // Tab 때 호출되는 리스너를 등록
        b.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                shuffle();
                startChronometer();
            }});
    }
    
    // （15）완성된 다이얼로그를 표시
    private void complete() {
        long msec = stopChronometer();
        AlertDialog.Builder b = new AlertDialog.Builder(this);
        b.setTitle(R.string.complete_title);
        b.setMessage(msec/1000 + " sec");
        b.setIcon(R.drawable.congratulations);
        b.setPositiveButton(
            R.string.complete_button,
            new DialogInterface.OnClickListener(){
                // 버튼이 눌러지면 다이얼로그를 닫는다
                public void onClick(DialogInterface dialog,int which) {
                    dialog.dismiss();
                }
            });
        b.show();
    }
}
