﻿/*
Copyright (c) 2009 Hideo KINAMI

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

// HelloService.java
package kr.hews.helloservice;

import android.app.Service;
import android.content.Intent;
import android.os.Handler;
import android.os.IBinder;
import android.util.Log;

// 서비스를 실행하는 클래스
public class HelloService extends Service implements Runnable {

    // 시작 ID
    private int mStartId;
    // 타이머 주기 3 초
    private static final int TIMER_PERIOD = 3 * 1000;	
    // 처리 횟수
    private static final int COUNT = 10;
    
    // 처리 카운터. 일정 조회 작업을 수행하면 완료
    private int mCounter;
    // 서비스에 대한 스레드에 연결된 Handler
    private Handler mHandler;
    // 동작 중 Frag
    private boolean mRunning;

    // 서비스를 생성할 때 호출
    public void onCreate() {
        Log.i("HelloService","HelloService created.");
        super.onCreate();
        mHandler = new Handler();
        mRunning = false;
    }


    // 서비스 시작 필요할 때 호출
    // 전달인자의 startId는 stopSelf에서 종료할 때 사용
    public void onStart(Intent intent, int startId) {
        Log.i("HelloService","onStart id = " + startId);
        super.onStart(intent, startId);
        mStartId = startId;
        mCounter = COUNT;
        // 이미 동작중이 아니면 run 메소드를 일정 시간 후에 시작
        if (! mRunning) {
            mHandler.postDelayed(this, TIMER_PERIOD);
            mRunning = true;
        }
    }	

    // 서비스의 종료시 호출
    public void onDestroy() {
        Log.i("HelloService","HelloService stop.");
        mRunning = false;
        super.onDestroy();
    }

    // 서비스 처리
    public void run() {
        Log.i("HelloService","run "+mCounter);

        if (! mRunning) {
            // onDestoryでサービスが停止していればなにもせずに終了
            Log.i("HelloService","run after destory");
            return;
        } else if (--mCounter <= 0) {
            // 지정한 횟수 실행하면 스스로 종료
            Log.i("HelloService","stop Service id = "+mStartId);
            stopSelf(mStartId);
        } else {
            // 다음 작업을 다시 요구
            mHandler.postDelayed(this, TIMER_PERIOD);
        }
    }

    // 원격 메소드 호출을 위해 사용
    // 메서드 호출을 제공하지 않으면 null을 반환
    public IBinder onBind(Intent intent) {
        return null;
    }
}
