﻿/*
Copyright (c) 2009 Hideo KINAMI

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

// HelloBindServiceActivity.java

package kr.hews.bindservice;

import java.util.Date;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.IBinder;
import android.os.RemoteException;
import android.util.Log;

public class HelloBindServiceActivity extends Activity {

    // 바인딩 서비스
    private IHelloService mService;
    // 마지막으로 등록한 index
    private int mLastIndex;

    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
    }

    protected void onStart() {
        super.onStart();
        // 서비스 바인딩
        Intent i = new Intent(this, HelloService.class);
        boolean ret = bindService(i,
                                  mConnection,
                                  Context.BIND_AUTO_CREATE);
        Log.i("HelloBindServiceActivity", "bindService  " + ret);
    }

    protected void onRestart() {
        super.onRestart();
        // 서비스 시간 등록
        putTimeStamp();
    }

    protected void onPause() {
        super.onPause();
        // 등록하는 시간을 가져
        getTimeStamp();
    }

    protected void onStop() {
        super.onStop();
        // 바인딩 종료
        unbindService(mConnection);
        Log.i("HelloBindServiceActivity", "unbindService");
    }

    // 서비스에 바인딩되어 있으면 시간 등록
    private void putTimeStamp() {
        if (mService != null) {
            try {
                mLastIndex = 
                    mService.addString((new Date()).toString());
            } catch (RemoteException e) {
                // 오류 처리
            }
        }
    }

    // 서비스에 바인딩되어 있으면 등록한 시간을 기록
    private void getTimeStamp() {
        if (mService != null) {
            try {
                String s = mService.getString(mLastIndex);
                Log.i("HelloBindServiceActivity","start time = " + s);
            } catch (RemoteException e) {
                // 오류 처리
            }
        }

    }

    private ServiceConnection mConnection = 
        new ServiceConnection() {

            // 서비스 바인딩 호출
            public void onServiceConnected(ComponentName name,
                                           IBinder service) {
                Log.i("HelloBindServiceActivity",
                      name.getShortClassName()+" connected.");
                // 인수 service를 IHelloService 인터페이스로 변환
                mService = IHelloService.Stub.asInterface(service);
                putTimeStamp();
            }

            // 서비스가 종료될 때 호출
            public void onServiceDisconnected(ComponentName name) {
                mService = null;
                Log.i("HelloBindServiceActivity",
                      name.getShortClassName()+" disconnected.");
            }
        };
}
