﻿/*
Copyright (c) 2009 Hideo KINAMI

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

// HelloService.java

package jp.hews.bindservice;

import java.util.ArrayList;
import java.util.List;

import android.app.Service;
import android.content.Intent;
import android.os.IBinder;
import android.os.RemoteException;

// 서비스 구현 클래스
public class HelloService extends Service {

    // 서비스를 요청할 때, IHelloService를 구현하는 인스턴스를 반환
    public IBinder onBind(Intent intent) {
        return mHelloService;
    }

    // IHelloService 구현. IHelloService.Stub을 상속
    private IHelloService.Stub mHelloService = new IHelloService.Stub() {

        // 데이터 저장
        private List<String> mStrs = new ArrayList<String>();

        // 문자열 추가, 추가된 index 반환
        public int addString(String str) throws RemoteException {
            mStrs.add(str);
            return mStrs.size()-1;
        }

        // 지정된 index 문자열을 검색
        public String getString(int index) throws RemoteException {
            // 인수를 확인하여 잘못된 경우 예외를 발생
            if (index < 0 || mStrs.size() < index)
                throw new RemoteException();
            return mStrs.get(index);
        }

        // 여러 index 문자열을 배열로 검색
        // 출력 배열의 개수를 요구보다 작으면 RemoteException을 생성
        public boolean getStrings(int[] indexes, String[] words)
            throws RemoteException {
            if (words.length < indexes.length)
                throw new RemoteException();
            for(int i=0; i<indexes.length; ++i) {
                if (indexes[i] < 0 || mStrs.size() < indexes[i])
                    return false;
                words[i] = mStrs.get(indexes[i]);
            }
            return true;
        }

        // 지정된 start에서 end의 한 단계 앞의 index 문자열 목록을 검색
        public List<String> getStringList(int start, int end)
            throws RemoteException {
            if (start < 0 || mStrs.size() < start ||
                end < 0 || mStrs.size() < end || end < start)
                throw new RemoteException();
            return mStrs.subList(start, end);
        }
    };
}
