﻿/*
Copyright (c) 2009 Hideo KINAMI

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

// HelloAlarmActivity.java
package kr.hews.helloalarm;

import java.util.Calendar;
import java.util.GregorianCalendar;

import android.app.Activity;
import android.app.AlarmManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.DatePicker;
import android.widget.TimePicker;
import android.widget.DatePicker.OnDateChangedListener;
import android.widget.TimePicker.OnTimeChangedListener;

public class HelloAlarmActivity extends Activity 
    implements OnDateChangedListener, OnTimeChangedListener {

    // 알람 매니저
    private AlarmManager mManager;

    // 설정 날짜
    private GregorianCalendar mCalendar;

    // 날짜 설정 클래스
    private DatePicker mDate;

    // 시간 설정 클래스
    private TimePicker mTime;

    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // 알람 매니저 취득
        mManager = (AlarmManager)
            getSystemService(Context.ALARM_SERVICE);

        // 현재 시간을 취득
        mCalendar = new GregorianCalendar();
        Log.i("HelloAlarmActivity",mCalendar.getTime().toString());

        // 설정 버튼, 리셋 버튼의 리스너를 등록
        setContentView(R.layout.main);
        Button b = (Button)findViewById(R.id.set);
        b.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                setAlarm();
            }
        });
        b = (Button)findViewById(R.id.reset);
        b.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                resetAlarm();
            }
        });

        // 날짜 설정 클래스에 현재 시간을 설정
        mDate = (DatePicker)findViewById(R.id.date_picker);
        mDate.init(
            mCalendar.get(Calendar.YEAR),
            mCalendar.get(Calendar.MONTH),
            mCalendar.get(Calendar.DAY_OF_MONTH),
            this);
        mTime = (TimePicker)findViewById(R.id.time_picker);
        mTime.setCurrentHour(mCalendar.get(Calendar.HOUR_OF_DAY));
        mTime.setCurrentMinute(mCalendar.get(Calendar.MINUTE));
        mTime.setOnTimeChangedListener(this);
    }

    // 알람 설정
    private void setAlarm() {
        mManager.set(AlarmManager.RTC_WAKEUP,
                     mCalendar.getTimeInMillis(),
                     pendingIntent());
        Log.i("HelloAlarmActivity",mCalendar.getTime().toString());
    }
    
    // 알람 해제
    private void resetAlarm() {
        mManager.cancel(pendingIntent());
    }

    // 알람 설정 시간에 발생하는 인텐트 만들기
    private PendingIntent pendingIntent() {
        Intent i = new Intent(getApplicationContext(),
                              HelloAlarmActivity.class); 
        PendingIntent pi = 
            PendingIntent.getActivity(this, 0, i, 0);
        return pi;
    }

    // 날짜 설정 클래스의 상태 변화 리스너
    public void onDateChanged(
        DatePicker view, int year, int monthOfYear, int dayOfMonth) {
        mCalendar.set(year, monthOfYear, dayOfMonth,
                      mTime.getCurrentHour(),
                      mTime.getCurrentMinute());
        Log.i("HelloAlarmActivity",mCalendar.getTime().toString());
    }

    // 시간 설정 클래스의 상태 변화 리스너
    public void onTimeChanged(
        TimePicker view, int hourOfDay, int minute) {
        mCalendar.set(
            mDate.getYear(),mDate.getMonth(),mDate.getDayOfMonth(),
            hourOfDay, minute);
        Log.i("HelloAlarmActivity",mCalendar.getTime().toString());
    }
}
