﻿/*
Copyright (c) 2009 Hideo KINAMI

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

// HelloPreferencesActivity.java
package kr.hews.hellopref;

import android.app.Activity;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.Bundle;
import android.widget.CheckBox;
import android.widget.EditText;

// Preferences 액세스 샘플
public class HelloPreferencesActivity extends Activity {

    // 외부로부터 액세스 할 수 있는 Preferences의 이름
    public static final String COMMON_PREF = "CommonPref";
    // 외부로부터 액세스 할 수 있는 Preferences에서 사용하는 키
    public static final String NAME = "Name";

    // Activity용의 Preferences에서 사용하는 키
    private static final String SILENT_MODE = "SilentMode";

    // Activity 생성 때에 Preferences에서 데이터를 검색
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
    }

    // Activity가 Foreground가 되었을 때에 데이터를 검색
    @Override
    protected void onResume() {
        super.onResume();

        // 외부에서 액세스 할 수 있는 Preferences에서 값을 검색
        SharedPreferences common = getSharedPreferences(
            COMMON_PREF,
            MODE_WORLD_READABLE | MODE_WORLD_WRITEABLE);
        EditText name = (EditText)findViewById(R.id.name);
        name.setText(common.getString(NAME, ""));

        // Activity용의 Preferences에서 값을 검색
        SharedPreferences activity = getPreferences(MODE_PRIVATE);
        CheckBox s = (CheckBox)findViewById(R.id.silent_mode);
        s.setChecked(activity.getBoolean(SILENT_MODE, false));		
    }

    // Activity의 정지(pause) 때에 데이터를 설정
    @Override
    protected void onPause() {
        super.onPause();

        // 외부에서 액세스 할 수 있는 Preference에 값을 설정
        SharedPreferences common = getSharedPreferences(
            COMMON_PREF,
            MODE_WORLD_READABLE | MODE_WORLD_WRITEABLE);
        EditText name = (EditText)findViewById(R.id.name);
        Editor editor = common.edit();
        editor.putString(NAME, name.getText().toString());
        editor.commit();

        // Activity용의 Preferences에 값을 설정
        SharedPreferences activity = getPreferences(MODE_PRIVATE);
        CheckBox s = (CheckBox)findViewById(R.id.silent_mode);
        editor = activity.edit();
        editor.putBoolean(SILENT_MODE, s.isChecked());
        editor.commit();
    }
}
