﻿/*
Copyright (c) 2009 Hideo KINAMI

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

package kr.hews.hellosqlite;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import android.app.Activity;
import android.content.ContentValues;
import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteException;
import android.database.sqlite.SQLiteOpenHelper;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;

public class DbInitializerActivity extends Activity implements OnClickListener {

    private SQLiteDatabase mDb;

    // 데이터베이스 열기 위한 도우미 클래스
    private static class MemoOpenHelper extends SQLiteOpenHelper {

        public MemoOpenHelper(Context c) {
            // 데이터베이스의 파일 이름과 버전을 지정
            super(c, HelloSQLiteActivity.DB, null, HelloSQLiteActivity.DB_VERSION);
        }

        // 데이터베이스를 새로 만듬
        public void onCreate(SQLiteDatabase db) {
            // 내부에 테이블 만들기
            db.execSQL(HelloSQLiteActivity.CREATE_TABLE);
        }

        // 존재하는 데이터베이스로 정의하고 있는 버전이 다를 때
        public void onUpgrade(SQLiteDatabase db, 
                              int oldVersion, int newVersion) {
            Log.w(HelloSQLiteActivity.TAG,
                  "Version mismatch :" + oldVersion + 
                  " to " + newVersion );
            // 여기에서는 테이블을 삭제하고 새로 만듬니다.
            // 일반적으로 테이블의 데이터 변환을 수행
            db.execSQL(HelloSQLiteActivity.DROP_TABLE);
            onCreate(db);
        }
    }

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.initializer);
        // 버튼 등록
        int buttons[] = {R.id.delete_db_button,
                R.id.add_db_button
        };
        for(int id: buttons) {
        	Button button = (Button)findViewById(id);
        	button.setOnClickListener(this);
        }
	}

	@Override
	protected void onResume() {
        MemoOpenHelper h = 
            new MemoOpenHelper(getApplicationContext());	
        try {
            mDb = h.getWritableDatabase();
        } catch (SQLiteException e) {
            // 디스크 전체 등에서 데이터를 쓸 수 없는 경우
            Log.e(HelloSQLiteActivity.TAG,e.toString());
            mDb = h.getReadableDatabase();
        }
		super.onResume();
	}

	@Override
	protected void onPause() {
		super.onPause();
		mDb.close();
	}


	private void writeInitialData() {
        InputStream in = getResources().openRawResource(
                R.raw.initialdata);
        try {
            // UTF - 8 텍스트 파일을 읽음
            BufferedReader reader = new BufferedReader(
                new InputStreamReader(in, "UTF-8"));
            String str;
            while( ( str = reader.readLine() ) != null ) {
            	ContentValues values = new ContentValues();
                values.put(HelloSQLiteActivity.MEMO, str);
                mDb.insert(HelloSQLiteActivity.TABLE, null, values);
                Log.i("HelloLocalFile", str);
            }
            reader.close();
        } catch (IOException e) {
            Log.e("HelloLocalFile", e.toString());
        }
	}

	public void onClick(View view) {
		
		if (view.getId() == R.id.add_db_button) {
			writeInitialData();
		} else if (view.getId() == R.id.delete_db_button) {
			mDb.execSQL(HelloSQLiteActivity.DROP_TABLE);
			mDb.execSQL(HelloSQLiteActivity.CREATE_TABLE);			
		}
		
	}

}
