﻿/*
Copyright (c) 2009 Hideo KINAMI

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

// HelloSQLiteActivity.java
package kr.hews.hellosqlite;

import android.app.ListActivity;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteException;
import android.database.sqlite.SQLiteOpenHelper;
import android.os.Bundle;
import android.provider.BaseColumns;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.SimpleCursorAdapter;
import android.widget.TextView;
import android.widget.Toast;

// 데이터베이스 액세스 샘플
public class HelloSQLiteActivity extends ListActivity
    implements OnClickListener
{
    // 데이터베이스 파일 이름
    static final String DB="daily_memo.db";

    // 테이블 이름
    static final String TABLE = "daily_memo";

    // 참고 열 이름
    static final String MEMO="memo";

    // 데이터베이스 버전
    static final int DB_VERSION=1;

    // 테이블 생성 SQL 
    static final String CREATE_TABLE =
        "create table daily_memo (" +
        "_id integer primary key autoincrement, " +
        "memo text not null );";
    // 테이블 삭제 SQL 
    static final String DROP_TABLE =
        "drop table daily_memo;";
    
    static final String TAG = "HelloSQLiteActivity";

    private SQLiteDatabase mDb;
    private TextView mCurrentId;
    private EditText mEdit;
    private SimpleCursorAdapter mAdapter;

    // 데이터베이스 열기위한 도우미 클래스
    private static class MemoOpenHelper extends SQLiteOpenHelper {

        public MemoOpenHelper(Context c) {
            // 데이터베이스의 파일 이름과 버전을 지정
            super(c, DB, null, DB_VERSION);
        }

        // 데이터베이스를 새로 만든 다음 호출
        public void onCreate(SQLiteDatabase db) {
            // 내부에 테이블 만들기
            db.execSQL(CREATE_TABLE);
        }

        // 존재하는 데이터베이스로 정의하고있는 버전이 다를 때
        public void onUpgrade(SQLiteDatabase db, 
                              int oldVersion, int newVersion) {
            Log.w(TAG,
                  "Version mismatch :" + oldVersion + 
                  " to " + newVersion );
            // 여기에서는 테이블을 삭제하고 새로 작성함
            // 일반적으로 테이블의 데이터 변환을 수행
            db.execSQL(DROP_TABLE);
            onCreate(db);
        }
    }

    // 작업의 시작 버튼 등록
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        mCurrentId = (TextView)findViewById(R.id.view_id);;
        mEdit = (EditText)findViewById(R.id.edit_memo);

        // 버튼 등록
        int buttons[] = {R.id.delete_button,
                         R.id.modify_button,
                         R.id.add_button
        };
        for(int id: buttons) {
            Button button = (Button)findViewById(id);
            button.setOnClickListener(this);
        }
        setEnabled(false);
    }
    
    // 작업이 포그라운드 된 시간에 데이터를 표시
    @Override
    protected void onResume() {
        super.onResume();

        // 데이터베이스를 쓰기 위한 열
        MemoOpenHelper h = 
            new MemoOpenHelper(getApplicationContext());
        try {
            mDb = h.getWritableDatabase();
        } catch (SQLiteException e) {
            // 디스크 전체에서 데이터를 쓸 수없는 경우
            Log.e(TAG,e.toString());
            mDb = h.getReadableDatabase();
        }

        // 커서를 가져갈 경우
        Cursor c = fetchAllMemo();

        // 기본 클래스 커서의 수명주기를 관리한다
        startManagingCursor(c);
        
        // 데이터베이스 컬럼과 뷰를 연결
        String[] from = new String[] { BaseColumns._ID, MEMO };
        int[] to = new int[] { R.id._id, R.id.memo_text };
        mAdapter = new SimpleCursorAdapter(
            this, R.layout.memo_row, c, from, to);
        setListAdapter(mAdapter);
    }

    // 작업이 포그 라운드 상태가 아닌 경우, DB를 close하기
    @Override
    protected void onPause() {
        super.onPause();
        mDb.close();
    }


    // 버튼을 눌렀을 때의 동작
    public void onClick(View view) {

        String id = mCurrentId.getText().toString();
        String str = mEdit.getText().toString();
        if (view.getId() == R.id.delete_button) {
            deleteMemo(id);
        } else if (view.getId() == R.id.modify_button) {
            if (str.length() != 0) {
                setMemo(id, str);
                toastMemo(str);
            } else {
                deleteMemo(id);
            }
        } else if (view.getId() == R.id.add_button) {
            if (str.length() != 0) {
                addMemo(str);
                toastMemo(str);
            }
        } else {
            Log.e(TAG, "Invalid button id");
        }

        // 커서를 다시 꺼내고 뷰를 다시 그리기
        mCurrentId.setText("");
        mEdit.setText("");
        setEnabled(false);
        Cursor c = fetchAllMemo();
        startManagingCursor(c);
        mAdapter.changeCursor(c);
    }

    // 목록 탭되었을 때 처리
    @Override
    protected void onListItemClick(
        ListView l, View v, int position, long id) {
        super.onListItemClick(l, v, position, id);

        // 탭된 행의 ID와 메모의 내용을 텍스트 편집 화면으로 설정
        LinearLayout ll = (LinearLayout)v;
        TextView t = (TextView)ll.findViewById(R.id.memo_text);
        mEdit.setText(t.getText());
        mCurrentId.setText(Long.toString(id));
        setEnabled(true);
        // 탭된 행의 내용 검색
        toastMemo(t.getText().toString());
    }

    // 전체 행 커서 ID의 내림차순으로 호출
    private Cursor fetchAllMemo() {
        return mDb.query(TABLE, new String[] {BaseColumns._ID, MEMO}, 
                         null, null, null, null, "_id DESC");
    }

    // 지정된 노트를 추가하여 ID를 반환
    private String addMemo(String memo) {
        ContentValues values = new ContentValues();
        values.put(MEMO, memo);
        long id = mDb.insert(TABLE, null, values);
        if (id < 0) {
            return "";
        }
        return Long.toString(id);
    }

    // 지정된 ID의 행 삭제
    private void deleteMemo(String id) {
        mDb.delete(TABLE, "_id = ?", new String[] { id });
    }

    // 지정된 ID 줄 메모 업데이트
    private void setMemo(String id, String memo) {

        // 업데이트 값 설정
        ContentValues values = new ContentValues();
        values.put(MEMO, memo);

        // 행을 업데이트
        mDb.update(
            TABLE,
            values,
            "_id = ?",
            new String[] { id });
    }

    private void toastMemo(String str) {
        if (str.length() == 0) {
            // 문서가 없을 때는 아무것도하지 않는다
            return;
        }
        String memo = searchMemo(str);
        Toast toast = Toast.makeText(
            getApplicationContext(), memo, Toast.LENGTH_LONG);
        toast.show();
    }

    // 지정된 문자열과 처음 1 문자가 같은 라인을 10 개까지 만회
    private String searchMemo(String str) {

        // 만회 값들을 지정
        final String[] projection = new String[] {
            BaseColumns._ID,
            MEMO
        };
        // projection의 컬럼 인덱스
        final int ID_INDEX = 0;
        final int MEMO_INDEX = 1;

        // 읽을 데이터의 조건
        String where = "memo like ?";
        // where의 ?을 대체할 매개 변수
        String param = str.substring(0, 1) + "%";

        // 검색
        Cursor c = mDb.query(
            TABLE,
            projection,
            where,  // WHERE
            new String[] { param },  // LIKE
            null,  // GROUP BY
            null,  // HAVING 
            "_id DESC",  // ORDERED BY 새로운 정렬
            "10");  // LIMIT 최대 10개

        StringBuffer buff = new StringBuffer();
        if (c.moveToFirst()) {
            // 검색 결과를 차례로 꺼내 문자열에 추가
            do {
                long id = c.getLong(ID_INDEX);
                buff.append("id(").append(id).append(") ");

                String memo = c.getString(MEMO_INDEX);
                buff.append(memo).append("\n");
            } while(c.moveToNext());
        }
        c.close();
        return buff.toString();
    }

    // 삭제, 수정 버튼 상태 변경
    private void setEnabled(boolean enabled) {
        int buttons[] = {R.id.delete_button,
                         R.id.modify_button
        };
        for(int id: buttons) {
            Button button = (Button)findViewById(id);
            button.setEnabled(enabled);
        }
    }
}
