﻿/*
Copyright (c) 2009 Hideo KINAMI

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

package kr.hews.hellocontent;

import java.io.FileNotFoundException;
import java.io.OutputStream;

import android.app.Activity;
import android.content.ContentResolver;
import android.content.ContentValues;
import android.content.res.Resources;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.Bundle;
import android.provider.Browser;
import android.provider.CallLog;
import android.provider.MediaStore;
import android.provider.Contacts.People;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

public class HelloContentsProviders extends Activity implements
		View.OnClickListener {

	private TextView mText;
	private EditText mEdit;

	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);
		mText = (TextView) findViewById(R.id.text_view);
		mEdit = (EditText) findViewById(R.id.edit_text);
		
		int ids[] =
			{R.id.call_log,
		     R.id.show_contacts,
		     R.id.show_bookmarks,
		     R.id.put_media};
		for(int i=0; i<ids.length; ++i) {
			Button b = (Button) findViewById(ids[i]);
			b.setOnClickListener(this);
		}
	}

	public void onClick(View v) {
		String str = mEdit.getText().toString();
		if (v.getId() == R.id.call_log) {
			int duration = 0;
			if (str.length() > 0) {
				duration = Integer.parseInt(str);
			}
			mText.setText(getCallLog(duration));
		} else if (v.getId() == R.id.show_contacts) {
			mText.setText(showContacts(str));
		} else if (v.getId() == R.id.show_bookmarks) {
			mText.setText(listAllBookmarks());
		} else if (v.getId() == R.id.put_media) {
			putMedia();
		}
		
		// String str = mEdit.getText().toString();
		// mText.setText("");
		// mText.append(getSearches());
		// mText.append(showContacts("H"));
		// mText.append("\n----\n");
		// mText.append(getCallLog(Integer.parseInt(str)));
		// mText.append("\n----\n");
		// mText.append(getMedia());
		// mText.append("\n----\n");
		// mText.append(addBookmark("Title",str).toString());
		// mText.append("\n----\n");
		// mText.append(listAllBookmarks());
		// mText.append("\n----\n");
	}

	private void putMedia() {
		Bitmap b = BitmapFactory.decodeResource(getResources(), R.drawable.icon);		
		putMedia("Icon" + new java.util.Date().toString(), b);
	}

	// 미디어스토어에 이미지를 저장하기
	private Uri putMedia(String name, Bitmap bitmap) {

		// 외부 미디어
		Uri media = MediaStore.Images.Media.EXTERNAL_CONTENT_URI;

		// 저장하는 이미지의 속성을 지정
		ContentValues values = new ContentValues();
		values.put(MediaStore.Images.Media.DISPLAY_NAME, name);
		values.put(MediaStore.Images.Media.MIME_TYPE, "image/jpeg");

		// 이미지를 저장할 공간을 확보하여 해당 Uri를 검색
		ContentResolver r = getContentResolver();
		Uri imageUri = r.insert(media, values);

		try {
			// 검색된 Uri에서 출력 스트림을 만회
			OutputStream out = r.openOutputStream(imageUri);
			// 출력 스트림에 데이터를 JPEG로 출력
			bitmap.compress(Bitmap.CompressFormat.JPEG, 50, out);
		} catch (FileNotFoundException e) {
			Log.e("HelloContentsProviders", e.toString());
			return null;
		}
		return imageUri;
	}


	// 북마크 URL를 읽고 1 개의 문자열로 조립
	private String listAllBookmarks() {

		// Browser 정의하는 도우미 메서드를 사용
		Cursor c = Browser.getAllBookmarks(getContentResolver());

		// 커서 사이의 URL을 저장하는 컬럼의 인덱스를 획득
		int index = c.getColumnIndex(Browser.BookmarkColumns.URL);
		StringBuffer buff = new StringBuffer();
		if (c.moveToFirst()) {
			do {
				// 커서에서 URL을 추출하여 문자열에 추가
				buff.append(c.getString(index));
				buff.append("\n");
			} while (c.moveToNext());
		}
		c.close();
		return buff.toString();
	}

	private String getCallLog(int duration) {

		// 통화 로그에서 만회 컬럼
		String[] projection = new String[] { CallLog.Calls.TYPE, // 발신, 수신 유형
				CallLog.Calls.NUMBER, // 전화 번호
				CallLog.Calls.DURATION // 통화 시간
		};
		// projection에서 지정한 컬럼 인덱스
		final int TYPE_INDEX = 0;
		final int NUMBER_INDEX = 1;
		final int DURATION_INDEX = 2;

		// 일정 시간 이상의 로그를 검색
		String selection = CallLog.Calls.DURATION + " >= " + duration;
		// 내림차순으로 정렬
		String order = CallLog.Calls.DURATION + " desc";

		// 검색 실행
		Cursor c = getContentResolver().query(CallLog.Calls.CONTENT_URI,
				projection, selection, null, order);
		// 결과가 없으면 null을 반환
		if (c == null)
			return "";

		// 검색 결과 1 개의 문자열로 조립
		StringBuffer buff = new StringBuffer();
		Resources res = getResources();
		if (c.moveToFirst()) {
			do {
				// 수신 또는 전송 여부를 확인
				if (c.getInt(TYPE_INDEX) == CallLog.Calls.INCOMING_TYPE) {
					buff.append(res.getString(R.string.incomming));
				} else { // CallLog.Calls.OUTGOING_TYPE
					buff.append(res.getString(R.string.outgoing));
				}
				buff.append(" : ");
				// 전화 번호
				buff.append(c.getString(NUMBER_INDEX));
				buff.append(" : ");
				// 통화 시간
				buff.append(c.getInt(DURATION_INDEX));
				buff.append("\n");
			} while (c.moveToNext());
		}
		c.close();
		return buff.toString();
	}

	private Cursor mCursor;

	// 이름에 지정한 문자가 포함된 연락처 목록 가져오기
	// android.permission.READ_CONTACTS 선언이 필요
	private String showContacts(String name) {

		// 연락처 목록에서 검색할 컬럼
		String[] projection = new String[] { People._ID, // ID
				People.NAME, // 이름
				People.NUMBER, // 전화 번호
		};

		// 활동 커서 수명주기 관리를 수행한다.
		mCursor = managedQuery(People.CONTENT_URI, projection, People.NAME
				+ " like ?", // 이름에 지정한 문자 포함
				new String[] { name + "%" }, People.NAME + " ASC"); // 이름으로 오름차순으로 정렬
		// 커서 작업

		StringBuffer buff = new StringBuffer();
		if (mCursor.moveToFirst()) {
			do {
				buff.append(mCursor.getLong(0));
				buff.append(":");
				buff.append(mCursor.getString(1));
				buff.append("  :");
				buff.append(mCursor.getString(2));
				buff.append("\n");
			} while (mCursor.moveToNext());
		}
		mCursor.close();
		return buff.toString();
	}
}
