﻿/*
Copyright (c) 2009 Hideo KINAMI

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

// HelloLocationActivity.java
package kr.hews.hellolocation;

import java.io.IOException;
import java.util.List;
import java.util.Locale;

import android.content.Context;
import android.location.Address;
import android.location.Geocoder;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.util.Log;
import android.view.Gravity;
import android.view.ViewGroup;
import android.widget.TextView;
import android.widget.ZoomControls;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;

public class HelloLocationActivity extends MapActivity
    implements LocationListener {

    // 기록 태그
    static final String TAG = "HelloLocationActivity";

    // 지도의 초기 값
    static final int INITIAL_ZOOM_LEVEL = 15;

    // 위치 정보 알림 영역
    static final long MIN_TIME=0; // 알림 최소 시간 [ms]
    static final float MIN_DISTANCE=0; // 알림 최단 거리 [m]

    // 위치 정보를 얻기 위해 매니저
    LocationManager mLocationManager;
    
    // 위치 변화시지도를 이동 컨트롤러
    MapController mMapController;

    // 지명 검색을 위한 클래스
    Geocoder mGeoCoder;

    // 지명 화면 표시 텍스트 뷰
    TextView mStatusText;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        // MapView 포함 레이아웃 설정
        setContentView(R.layout.main);

        // MapView에서 컨트롤러를 검색
        MapView map_view = (MapView) findViewById(R.id.mapview);

        // 지도의 확대를 제어하는 줌 컨트롤러를 배치
        ZoomControls zoom = (ZoomControls) map_view.getZoomControls();
        ViewGroup.LayoutParams layout = new ViewGroup.LayoutParams(
            ViewGroup.LayoutParams.FILL_PARENT,
            ViewGroup.LayoutParams.FILL_PARENT);
        zoom.setLayoutParams(layout);
        zoom.setGravity(Gravity.BOTTOM | Gravity.CENTER_HORIZONTAL);
        map_view.addView(zoom);

        // 위치와 확대 수준의 초기 상태를 설정
        mMapController = map_view.getController();
        mMapController.setZoom(INITIAL_ZOOM_LEVEL);

        // 지명 검색을 위한 클래스
        mGeoCoder = 
            new Geocoder(getApplicationContext(), Locale.JAPAN);
        mStatusText = (TextView)findViewById(R.id.status_text);

        // 위치 정보 검색을위한 클래스
        mLocationManager = (LocationManager)getSystemService(
            Context.LOCATION_SERVICE);
    }

    @Override
    protected void onResume() {
        super.onResume();
        // 마지막으로 얻은 위치가 있으면 보기
        Location loc = mLocationManager.getLastKnownLocation(
            LocationManager.GPS_PROVIDER);
        if (loc != null) {
            onLocationChanged(loc);
        }
        // 현재 위치가 변화하는 경우 메서드가 호출되도록 등록
        mLocationManager.requestLocationUpdates(
            LocationManager.GPS_PROVIDER,
            MIN_TIME,
            MIN_DISTANCE,
            this);
    }

    @Override
    protected void onPause() {
        super.onPause();
        // 위치 정보 통지 등록 취소
        mLocationManager.removeUpdates(this);
    }

    // 경로 정보를 제공하지 않기 때문에 false를 반환
    @Override
    protected boolean isRouteDisplayed() {
        return false;
    }

    public void onLocationChanged(Location location) {
        // Location을 GeoPoint 변경하여지도보기 변경
        GeoPoint gp = 
            new GeoPoint((int)(location.getLatitude()*1E6),
                         (int)(location.getLongitude()*1E6));
        mMapController.animateTo(gp);

        // 현재 지명을 표시하는
        showLocationName(location);
    }

    public void onProviderDisabled(String provider) {
        // 공급자 사용 불가능
    }

    public void onProviderEnabled(String provider) {
        // 공급자 사용 가능
    }

    public void onStatusChanged(String provider, int status,
                                Bundle extras) {
        // 공급자 상태 변화
    }


    // GeoPoint 위치의 지명을 표시
    private void showLocationName(Location loc) {
        // 좌표 변환
        double latitude = loc.getLatitude();
        double longitude = loc.getLongitude();
        // Geocoder를 사용하여 지명 검색
        StringBuffer buff = new StringBuffer();
        try {
            List<Address> addrs =
                mGeoCoder.getFromLocation(latitude, longitude, 1);

            for (Address addr : addrs) {

                // 지명을 검색하고 문자열에 연결
                int index = addr.getMaxAddressLineIndex();
                for (int i = 0; i <= index; ++i) {
                    buff.append(addr.getAddressLine(i));
                    buff.append(" ");
                }
                buff.append("\n");
            }
        } catch (IOException e) {
            Log.e(TAG, e.toString());
        }
        // 텍스트 보기
        mStatusText.setText(buff.toString());
    }
}
