/*
Copyright (c) 2009 Hideo KINAMI

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

package kr.hews.sensorsample;

import org.openintents.hardware.SensorManagerSimulator;
import org.openintents.provider.Hardware;

import android.app.Activity;
import android.content.Intent;
import android.content.res.Resources;
import android.hardware.SensorListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.widget.TextView;

public class SensorSample extends Activity {

	SensorListener mSensorListener = new SensorListener() {

		private int mOriValCount;
		private int mOriAccCount;
		private int mAccValCount;
		private int mAccAccCount;
		
		public void onAccuracyChanged(int sensor, int accuracy) {
			if (sensor == SensorManager.SENSOR_ORIENTATION) {
				++mOriAccCount;
			} else if (sensor == SensorManager.SENSOR_ACCELEROMETER) {
				++mAccAccCount;
			}
			setCount();
		}

		public void onSensorChanged(int sensor, float[] values) {
			if (sensor == SensorManager.SENSOR_ORIENTATION) {
				++mOriValCount;
			} else if (sensor == SensorManager.SENSOR_ACCELEROMETER) {
				++mAccValCount;
			}
			setCount();			
		}
		
		private void setCount() {
	    	StringBuffer buff = new StringBuffer();
	    	buff.append("mOriValCount:").append(mOriValCount).append("\n");
	    	buff.append("mOriAccCount:").append(mOriAccCount).append("\n");
	    	buff.append("mAccValCount:").append(mAccValCount).append("\n");
	    	buff.append("mAccAccCount:").append(mAccAccCount).append("\n");
	    	TextView tv = (TextView)findViewById(R.id.counters);
	    	tv.setText(buff.toString());
		}
	};
	
	private SensorManager mManager;
	private boolean mSimulator;
	
	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        setupManager();
    }
    
    

	@Override
	protected void onResume() {
		if (mSimulator)
			SensorManagerSimulator.connectSimulator();
		listSensors();
		mManager.registerListener(
			mSensorListener,
			SensorManager.SENSOR_ORIENTATION |
			SensorManager.SENSOR_ACCELEROMETER);
		super.onResume();
	}

    
    @Override
	protected void onStop() {
    	mManager.unregisterListener(
    			mSensorListener,
    			SensorManager.SENSOR_ORIENTATION |
    			SensorManager.SENSOR_ACCELEROMETER);
		if (mSimulator)
			SensorManagerSimulator.disconnectSimulator();
		super.onStop();
	}

	private void setupManager() {
    	mManager = (SensorManager)getSystemService(SENSOR_SERVICE);
    	if (mManager.getSensors() != 0) {
    		// Real Device (Dev Phone)
    		mSimulator = false;
    		return;
    	}
    	// Emulator, Use OpenIntnet Simulator
    	mSimulator = true;
    	Hardware.mContentResolver = getContentResolver(); 
    	mManager = new SensorManagerSimulator(mManager);
    	startActivity(new Intent(Intent.ACTION_VIEW,
    					Hardware.Preferences.CONTENT_URI));
    	SensorManagerSimulator.connectSimulator();
    }

    
    private void listSensors() {
    	Resources res = getResources();
    	StringBuffer buff = new StringBuffer();
    	int sensors = mManager.getSensors();
    	buff.append(sensors);
    	buff.append("\n");
    	if ((sensors & SensorManager.SENSOR_ORIENTATION) != 0) {
    		buff.append(res.getString(R.string.sensor_orientation));
    		buff.append("\n");
    	}
    	if ((sensors & SensorManager.SENSOR_ORIENTATION_RAW) != 0) {
    		buff.append(res.getString(R.string.sensor_orientation_raw));
    		buff.append("\n");
    	}
    	if ((sensors & SensorManager.SENSOR_ACCELEROMETER) != 0) {
    		buff.append(res.getString(R.string.sensor_accelerometer));
    		buff.append("\n");
    	}
    	if ((sensors & SensorManager.SENSOR_TEMPERATURE) != 0) {
    		buff.append(res.getString(R.string.sensor_temperature));
    		buff.append("\n");
    	}
    	if ((sensors & SensorManager.SENSOR_MAGNETIC_FIELD) != 0) {
    		buff.append(res.getString(R.string.sensor_magnetic_field));
    		buff.append("\n");
    	}
    	if ((sensors & SensorManager.SENSOR_LIGHT) != 0) {
    		buff.append(res.getString(R.string.sensor_light));
    		buff.append("\n");
    	}
    	if ((sensors & SensorManager.SENSOR_PROXIMITY) != 0) {
    		buff.append(res.getString(R.string.sensor_proximity));
    		buff.append("\n");
    	}
    	if ((sensors & SensorManager.SENSOR_TRICORDER) != 0) {
    		buff.append(res.getString(R.string.sensor_tricorder));
    		buff.append("\n");
    	}
    	TextView tv = (TextView)findViewById(R.id.sensors);
    	tv.setText(buff.toString());
    }
    
}