﻿/*
Copyright (c) 2009 Hideo KINAMI

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

// HelloCamera.java
package kr.hews.hellocamera;

import android.app.Activity;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.hardware.Camera;
import android.os.Bundle;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.widget.ImageButton;
import android.widget.ImageView;

/**
 * 카메라 샘플
 */
public class HelloCamera extends Activity {

    // Log용 TAG
    private static final String TAG = "HelloCamera";
    // ImageView 설정 이미지 약하게 크기. 1 / 8의 크기로 처리
    private static final int IN_SAMPLE_SIZE = 8;

    // 카메라 제어
    private Camera mCamera;
    // 촬영 사진보기
    private ImageView mImage;
    // 처리중 신고
    private boolean mInProgress;

    // 카메라 미리보기 SurfaceView의 리스너
    private SurfaceHolder.Callback mSurfaceListener = 
        new SurfaceHolder.Callback() {

            public void surfaceCreated(SurfaceHolder holder) {
                // SurfaceView가 생성되면 카메라를 열
                mCamera = Camera.open();
                Log.i(TAG, "Camera opened");
                // SDK1.5에서 setPreviewDisplay이 IO Exception을 throw한다
                try {
					mCamera.setPreviewDisplay(holder);
				} catch (Exception e) {
					e.printStackTrace();
				}
            }

            public void surfaceDestroyed(SurfaceHolder holder) {
                // SurfaceView가 삭제되는 시간에 카메라를 개방
                mCamera.release();
                mCamera = null;
                Log.i(TAG, "Camera released");
            }

            public void surfaceChanged(SurfaceHolder holder,
                                       int format,
                                       int width,
                                       int height) {
                // 미리보기 크기를 설정
                Camera.Parameters parameters =
                    mCamera.getParameters();
                parameters.setPreviewSize(width, height);
                mCamera.setParameters(parameters);
                mCamera.startPreview();
                Log.i(TAG, "Camera preview started");
            }
        };

    // 버튼을 눌렀을 때 수신기
    private View.OnClickListener mButtonListener =
        new View.OnClickListener() {

            public void onClick(View v) {
                if (mCamera != null && mInProgress == false) {
                    // 이미지 검색을 시작한다. 리스너 설정
                    mCamera.takePicture(
                        mShutterListener,  // 셔터 후
                        null, // Raw 이미지 생성 후
                        mPicutureListener); // JPE 이미지 생성 후
                    mInProgress = true;
                }
            }

        };


    private Camera.ShutterCallback mShutterListener =
        new Camera.ShutterCallback() {

            // 이미지를 처리하기 전에 호출된다.
            public void onShutter() {
                // 셔터 소리 재생 코드 생략
                Log.i(TAG, "onShutter");
            }

        };

    // JPEG 이미지를 생성 후 호출
    private Camera.PictureCallback mPicutureListener =
        new Camera.PictureCallback() {

            public void onPictureTaken(byte[] data, Camera camera) {
                Log.i(TAG, "Picture Taken");
                if (data != null) {
                    Log.i(TAG, "JPEG Picture Taken");
				
                    // 처리하는 이미지의 크기를 축소
                    BitmapFactory.Options options =
                        new BitmapFactory.Options();
                    options.inSampleSize = IN_SAMPLE_SIZE;
                    Bitmap bitmap =
                        BitmapFactory.decodeByteArray(data,
                                                      0,
                                                      data.length,
                                                      options);
                    //이미지 뷰 이미지 설정
                    mImage.setImageBitmap(bitmap);
                    // 정지된 프리뷰를 재개
                    camera.startPreview();
                    // 처리중 플래그를 떨어뜨림
                    mInProgress = false;
                }
            }

        };

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        mImage = (ImageView) findViewById(R.id.image_view);

        SurfaceView surface =
            (SurfaceView) findViewById(R.id.surface_view);
        SurfaceHolder holder = surface.getHolder();

        // SurfaceView 리스너를 등록
        holder.addCallback(mSurfaceListener);
        // 외부 버퍼를 사용하도록 설정
        holder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);

        // 셔터의 버튼에 리스너를 등록
        ImageButton button = (ImageButton) findViewById(R.id.shutter);
        button.setOnClickListener(mButtonListener);
    }
}
