﻿/*
   Copyright 2009 adamrocker

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

// CameZonActivity.java
package com.adamrocker.android.camezon;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.view.Window;
import android.view.WindowManager;

// 이미지 미리보기에서 바코드를 취득 Amazon에서 검색을 실행한다.
public class CameZonActivity extends Activity {

    final static String TAG = "CameZon";
    private static final int INCORRECT_BARCODE_TYPE = 0;

    private FinderView mFinderView;
    private Preview mPreview;
    //메서드 간의 전달을 위해서만 사용
    private String mBarcodeType;
    //바코드가 구문 분석되지 않은 경우 표시되는 대화 상자에 대한 설명
    private String mIncorrectBarcodeError;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // 화면을 밝은 상태로 유지
        Window window = getWindow();
        window.addFlags(
            WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        requestWindowFeature(Window.FEATURE_NO_TITLE);

        setContentView(R.layout.camezon);

        // 미리 용 측정기 보기
        mFinderView = (FinderView) findViewById(R.id.viewfinder_view);
        mPreview = (Preview) findViewById(R.id.preview_surface);
        mPreview.setFinder(mFinderView);
		
        mIncorrectBarcodeError = 
            getResources().getString(R.string.incorrect_barcode_error);
    }

    void requestAutoFocus() {
        mPreview.requestAutoFocus();
    }

    @Override
    protected void onStop() {
        super.onStop();
        mPreview.closeCamera();
    }

    // 바코드의 내용을 분석할 수있는 경우 대화 상자 표시
    void showIncorrectDialog(String barcodeType) {
        mBarcodeType = barcodeType;
        showDialog(INCORRECT_BARCODE_TYPE);
    }

    // 대화 상자의 표시. 버튼을 누르면 다시 분석 시작
    @Override
    protected Dialog onCreateDialog(int id) {
        switch (id) {
          case INCORRECT_BARCODE_TYPE:
            return 
                new AlertDialog.Builder(CameZonActivity.this).setIcon(
                    R.drawable.alert_dialog_icon).setTitle(
                        mBarcodeType + " : " + mIncorrectBarcodeError)
                .setPositiveButton(R.string.alert_dialog_ok,
                  new DialogInterface.OnClickListener() {
                      public void onClick(DialogInterface dialog,
                                          int whichButton) {
                          // 자동 초점 시작
                          mPreview.requestAutoFocus();
                      }
                  }).setNegativeButton(R.string.alert_dialog_cancel,
                       new DialogInterface.OnClickListener() {
                           public void onClick(DialogInterface dialog,
                                               int whichButton) {
                               // 자동 초점 시작
                               mPreview.requestAutoFocus();
                           }
                       }).create();
        }
        return null;
    }
}